/*------------------------------------------------------------------------
Bullet.cpp: implementation of the CBullet class.
DroidWars Video Game - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Examine the license.txt file located in the main source code directory
    to see the full text of the GPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:  http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/

#include "Bullet.h"
#include "Droid.h"


CBullet::CBullet()
{
    Initialize();  // Is this even needed?  -TDP
}



CBullet::~CBullet()
{
}




int CBullet::Render(CGridView *gv, unsigned long lClockTicks,
    int nScreenX, int nScreenY)
{
    switch ( m_ObjType ) {

    case OT_FIREBALL:
    case OT_MISSILE:
        // Draw the bullet
        gv->DrawPic( m_PicNum, nScreenX+20, nScreenY+5);
        // Draw its shadow
        gv->DrawPic( 23, nScreenX+20, nScreenY+40);
        break;
    case OT_SMOKE:
    case OT_EXPLOSION:
        gv->DrawPic( m_PicNum, nScreenX, nScreenY - 10);
        break;
    }        
    return 0;
}





int CBullet::Process(unsigned long lClockTicks, double dTimeSlice)
{
    CGridObj* p;

    double newx, newy;

    // Get the new X and Y position
    newx = m_XPos + m_XSpeed * dTimeSlice;
    newy = m_YPos + m_YSpeed * dTimeSlice;

    switch (m_ObjType) {
    case OT_FIREBALL:
        // Did we collide with anything?
        if ( CollisionCheck(newx, newy, &p) ) {

            if (p && p->m_ObjType == OT_TANK) {
                ((CDroid *) p)->ChangeArmor( -5 );
                ((CDroid *) p)->ChangeHeat( 200 );
                if (((CDroid *) p)->m_Armor < 0) {
                    p->DestroyObject();
                    CreateExplosion(m_XPos-0.5, m_YPos-0.5);
                    CreateExplosion(m_XPos, m_YPos-0.5);
                    CreateExplosion(m_XPos-0.5, m_YPos);
                    CreateExplosion(m_XPos+0.5, m_YPos+0.5);
                }
            }

            // Create an explosion
            CreateExplosion(m_XPos, m_YPos);
            PlaySound(1);

            // Delete the Bullet
            DestroyObject();
        }
        else {
            // Remove the old grid reference
            RemoveObjFromSquare();

            // don't let it go off the edge
            // This should never happen in a properly built grid.
            if (newx < 0.0) newx = 0.0;
            else if (newx > (float) MAX_X_GRID_SIZE - 0.1) newx = 99.9;
            m_XPos = newx;
            if (newy < 0.0) newy = 0.0;
            else if (newy > (float) MAX_Y_GRID_SIZE - 0.1) newy = 99.9;
            m_YPos = newy;

            // Create a new grid reference to the object
            AddObjToSquare();
        }
        break;
    case OT_EXPLOSION:
        m_LifeSpan++;
        m_PicNum = 17 + m_LifeSpan / 3;
        if (m_PicNum > 21 ) {
            DestroyObject();
        }
        break;
    }

    return 0;
}




void CBullet::CreateExplosion(double xpos, double ypos)
{
    GGridInfo info;

    // Create an explosion
    CBullet* pExplosion = new CBullet();
    pExplosion->ConnectToGrid(FillGridInfo(&info));
    if (pExplosion == 0) return;
    pExplosion->m_ObjType = OT_EXPLOSION;
    pExplosion->m_PicNum = 17;
    pExplosion->m_LifeSpan = 0;
    pExplosion->m_Speed = 0.0;
    pExplosion->m_Direction = 0.0;
    pExplosion->m_XPos = xpos;
    pExplosion->m_YPos = ypos;
    pExplosion->m_Solid = 0;
    pExplosion->AddObjToSquare();
}
