/*------------------------------------------------------------------------
Droid.cpp: implementation of the CDroid class.
DroidWars Video Game - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Examine the license.txt file located in the main source code directory
    to see the full text of the GPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/

#include "Droid.h"
#include "Bullet.h"
#include "Door.h"
#include <stdlib.h>


CDroid::CDroid()
{
    Initialize();
}



CDroid::~CDroid()
{
}



void CDroid::Initialize( void )
{
    CGridActor::Initialize();
    m_Armor = m_MaxArmor = 100;
    m_Ammo = m_MaxAmmo = 100;
    m_Fuel = m_MaxFuel = 10000;
    m_Heat = 0;
    m_MaxHeat = 1000;
    m_KeyCount = 0;
    m_ArmorPic = 0;
    m_AmmoPic = 0;
    m_FuelPic = 0;
    m_HeatPic = 0;
    m_MaxSpeed = 0.15;
    m_WeaponRecharge = 20;
    m_WeaponStatus = 0;
    m_HeatSink = -5;
}




int CDroid::Render(CGridView *gv, unsigned long lClockTicks,
    int nScreenX, int nScreenY)
{
    int picnum;
    int picoffset;
    int xtmp;
    int ytmp;
    int nEffect;
    // Ugly Hack Alert!  These should not be harcoded.  -TDP
    int NorthXFactor = 23;
    int NorthYFactor = -14;
    int EastXFactor  = -26;
    int EastYFactor  = -12;

    // this is a bletcherous hack!
    m_ArmorPic = gv->GetPic(24);  // bletch! -TDP
    m_AmmoPic = gv->GetPic(26);  // bletch! -TDP
    m_FuelPic = gv->GetPic(27);  // bletch! -TDP
    m_HeatPic = gv->GetPic(28);  // bletch! -TDP

    if (IsSelected()) nEffect = DP_HILIGHT;
    else nEffect = DP_FULL;

    if (m_PicNum == 6) {
        gv->DrawPic( m_PicNum, nScreenX, nScreenY-8, nEffect);
        gv->DrawPic( 49, nScreenX, nScreenY+31, DP_SHADOW );
    }
    else {
        // Draw the body of the Actor
        picoffset = abs( ((int) (m_Direction * 10.0)) % 2 );
        picnum = m_PicNum + picoffset;
        gv->DrawPic( picnum, nScreenX, nScreenY-5, nEffect);

        // Draw the antenna
        xtmp = (int) ( ((double) NorthXFactor) *
                   CalcXSpeed( 0.52, m_Direction ) +
                   ((double) EastXFactor) *
                   CalcYSpeed( 0.52, m_Direction ) );
        ytmp = (int) ( ((double) NorthYFactor) *
                   CalcXSpeed( 0.52, m_Direction ) +
                   ((double) EastYFactor) *
                   CalcYSpeed( 0.52, m_Direction ) );

        if (m_Direction > 3.35 && m_Direction < 4.45) {
            gv->DrawPic( 41, nScreenX+24+xtmp, nScreenY+6+ytmp, nEffect);
        }
        else if (m_Direction > 2.85 && m_Direction < 4.85) {
            gv->DrawPic( 42, nScreenX+24+xtmp, nScreenY+6+ytmp, nEffect);
       }
        else if (m_Direction > 2.50 && m_Direction < 5.15) {
            gv->DrawPic( 43, nScreenX+24+xtmp, nScreenY+6+ytmp, nEffect);
        }
        else if (m_Direction > 2.3 && m_Direction < 5.42) {
            gv->DrawPic( 44, nScreenX+24+xtmp, nScreenY+7+ytmp, nEffect);
        }

        xtmp = (int) ( ((double) NorthXFactor) *
                   CalcXSpeed( 0.32, m_Direction ) +
                   ((double) EastXFactor) *
                   CalcYSpeed( 0.32, m_Direction ) );
        ytmp = (int) ( ((double) NorthYFactor) *
                   CalcXSpeed( 0.32, m_Direction ) +
                   ((double) EastYFactor) *
                   CalcYSpeed( 0.32, m_Direction ) );

        if (m_Team == 0) {
            gv->DrawPic( 46, nScreenX+21-xtmp, nScreenY-16-ytmp, nEffect);
        }
        else {
            gv->DrawPic( 45, nScreenX+21-xtmp, nScreenY-16-ytmp, nEffect);
        }
    }

    return 0;
}





int CDroid::Process(unsigned long lClockTicks, double dTimeSlice)
{
    int maxwidth = 75;

    if (m_WeaponStatus) m_WeaponStatus--;  // adjust for timeslice! -TDP

    if (m_Team == 0) {
        CGridActor::Process(lClockTicks, dTimeSlice);
        CDroid* p = RadarSweep(20.0);
        if (p) {
            double dir = CalcDirection( p->m_XPos-m_XPos, p->m_YPos-m_YPos );
            m_Direction = dir;
            if (m_Direction < 0.0)
                m_Direction += TWO_PI_RADIANS;
            if (m_Direction > TWO_PI_RADIANS)
                m_Direction -= TWO_PI_RADIANS;
            m_XSpeed = CalcXSpeed( m_Speed, m_Direction );
            m_YSpeed = CalcYSpeed( m_Speed, m_Direction );
            if ( !(rand()%200) ) FireWeapon(1);
        }
        else {
            m_Direction += 0.01;
            if (m_Direction < 0.0)
                m_Direction += TWO_PI_RADIANS;
            if (m_Direction > TWO_PI_RADIANS)
                m_Direction -= TWO_PI_RADIANS;

            m_XSpeed = CalcXSpeed( m_Speed, m_Direction );
            m_YSpeed = CalcYSpeed( m_Speed, m_Direction );
        }
    }
    else {
        double newx = m_XPos + m_XSpeed * dTimeSlice;
        double newy = m_YPos + m_YSpeed * dTimeSlice;
        CGridObj* p;

        // Did we collide with anything?
        if ( CollisionCheck(newx, newy, &p) ) {
            if (p) {
                switch(p->m_ObjType) {
                case OT_DOOR:
                    if (TryKeys(p->m_Value))
                        ((CDoor *) p)->MoveDoor(-500);
                    break;
                case OT_KEY:
                    AddKey(p->m_Value);
                    p->DestroyObject();
                    break;
                case OT_FUEL:
                    m_Fuel = m_MaxFuel;
                    p->DestroyObject();
                    break;
                case OT_AMMO:
                    m_Ammo = m_MaxAmmo;
                    m_AmmoPic->left = 421 - m_Ammo * maxwidth / m_MaxAmmo ;
                    m_AmmoPic->right = m_AmmoPic->left + 75;
                    p->DestroyObject();
                    break;
                case OT_ARMOR:
                    m_Armor = m_MaxArmor;
                    m_ArmorPic->left = 421 - m_Armor * maxwidth / m_MaxArmor ;
                    m_ArmorPic->right = m_ArmorPic->left + 75;
                    p->DestroyObject();
                    break;
                }
            }
        }
        if (m_FuelPic) {
            double speedsign;
            if (m_Speed >= 0.0) speedsign = 1.0;
            else speedsign = -1.0;
            m_Fuel -= (int) (m_Speed * 20.0 * speedsign);
            if (m_Fuel < 0) {
                m_Fuel = 0;
                m_Speed = 0.02 * speedsign;
                ChangeDirection( 0.0 );
            }
            if (m_Fuel > m_MaxFuel) m_Fuel = m_MaxFuel;
            m_FuelPic->left = 421 - m_Fuel * maxwidth / m_MaxFuel ;
            m_FuelPic->right = m_FuelPic->left + 75;
        }
        if (m_HeatPic) {
            ChangeHeat( m_HeatSink );
            if (m_Heat > m_MaxHeat) {
                //CreateExplosion(m_XPos-0.5, m_YPos-0.5);
                //CreateExplosion(m_XPos, m_YPos-0.5);
                //CreateExplosion(m_XPos-0.5, m_YPos);
                //CreateExplosion(m_XPos+0.5, m_YPos+0.5);
                DestroyObject();
            }
            else {
                m_HeatPic->left = 421 - m_Heat * maxwidth / m_MaxHeat;
                m_HeatPic->right = m_HeatPic->left + 75;
            }
        }
        CGridActor::Process(lClockTicks, dTimeSlice);
    }

    return 0;
}






void CDroid::FireWeapon( int nGun )
{
    CBullet* pBullet;
    int maxwidth = 75;
    GGridInfo info;

    if (m_WeaponStatus) return;
    m_WeaponStatus = m_WeaponRecharge;

    if (m_Team == 1) {
        if (m_Ammo <= 0 ) return;
        m_Ammo--;
        if (m_AmmoPic) {
            m_AmmoPic->left = 421 - m_Ammo * maxwidth / m_MaxAmmo ;
            m_AmmoPic->right = m_AmmoPic->left + 75;
        }
    }

    //pBullet = CreateNewObject( OT_FIREBALL );
    pBullet = new CBullet();

    pBullet->ConnectToGrid(FillGridInfo(&info));

    if (pBullet == 0) return;

    // this all should be done through a method  -TDP
    pBullet->m_ObjType = OT_FIREBALL;
    pBullet->m_PicNum = 16;
    pBullet->m_LifeSpan = 50;

    pBullet->m_Diameter = 0.1;
    pBullet->m_Radius = 0.05;

    pBullet->m_Speed = 0.5;
    pBullet->m_Direction = m_Direction;
    pBullet->m_XSpeed =
        CalcXSpeed(pBullet->m_Speed,
        pBullet->m_Direction);
    pBullet->m_YSpeed =
        CalcYSpeed(pBullet->m_Speed,
        pBullet->m_Direction);
    pBullet->m_XPos = m_XPos +
        ( pBullet->m_XSpeed * 1.2 );
    pBullet->m_YPos = m_YPos +
        ( pBullet->m_YSpeed * 1.2 );

    pBullet->AddObjToSquare();
    
    PlaySound( 0 );

    return;
}




CDroid* CDroid::RadarSweep( double dDistance )
{
    CDroid* p;
    double x,y;

    p = (CDroid *) m_Objects[0];  // ugly hack alert! -TDP
    if (p->m_Team != 1) return 0;
    x = p->m_XPos-m_XPos;
    y = p->m_YPos-m_YPos;
    if ( x*x + y*y < dDistance*dDistance ) return p;

    return 0;
}




int CDroid::ChangeArmor( int nChange )
{
    int maxwidth = 75;

    m_Armor += nChange;
    if (m_Team == 1) {
        if (m_ArmorPic) {
            m_ArmorPic->left = 421 - m_Armor * maxwidth / m_MaxArmor ;
            m_ArmorPic->right = m_ArmorPic->left + 75;
        }
    }

    return 0;
}





void CDroid::ChangeSpeed( double change )
{
    if (m_Speed + change > m_MaxSpeed ||
        m_Speed + change < -m_MaxSpeed) return;
    m_Speed += change;
    ChangeDirection( 0.0 );
}




void CDroid::ChangeHeat( int change )
{
    m_Heat += change;
    if (m_Heat < 0) m_Heat = 0;
}




void CDroid::AddKey( int keynum )
{
    if (m_KeyCount<MAX_KEYS) m_KeyList[m_KeyCount++] = keynum;
}




int CDroid::TryKeys( int locknum )
{
    for (int k=0; k<m_KeyCount; k++) {
        if (m_KeyList[k] == locknum) return 1;
    }
    return 0;
}
