/*------------------------------------------------------------------------
DroidWars.cpp: Example video game for the GridSlammer toolkit
DroidWars Video Game - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Examine the license.txt file located in the main source code directory
    to see the full text of the GPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/
#define NAME "DroidWars"
#define TITLE "DroidWars"

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include "resource.h"
#include "gridslammer/GridStage.h"
#include "Droid.h"
#include "Door.h"
#include "GridMenu.h"
//#include "GridSlammer\GridActor.h"

CGridStage GridStage;
CGridView  GridView;
CGridMenu  Menu;
CDroid *MyDroid;
//CGridActor *MyDroid;


int LoadGridFile( char *filename );


long FAR PASCAL WindowProc( HWND hWnd, UINT message, 
                            WPARAM wParam, LPARAM lParam )
{
    int nXpos;
    int nYpos;

    switch( message )
    {
    case WM_ACTIVATEAPP:
        GridView.SetActive(wParam);
        break;

    case WM_SETCURSOR:
        SetCursor(NULL);
        return TRUE;

    case WM_CREATE:
        break;

    case WM_KEYDOWN:
        if (Menu.IsActive()) {
            switch( wParam )
            {
            case VK_ESCAPE:
            case VK_F12:
                Menu.Close();
                break;
            case VK_UP:
                Menu.ArrowUp();
                break;
            case VK_DOWN:
                Menu.ArrowDown();
                break;
            case VK_SPACE:
            case VK_RETURN:
                switch(Menu.Select()) {
                    case 0:    break;
                    case 1:    break;
                    case 2:    break;
                    case 3:    break;
                    case 4:    break;
                    case 5:    PostMessage(hWnd, WM_CLOSE, 0, 0); break;
                }
                break;
            }
        }
        else {
            switch( wParam )
            {
            case VK_ESCAPE:
            case VK_F12:
                // Render the stage twice so the same image is in
                // both frames, otherwise the screen will *flicker*
                // while the menu is up.
                GridStage.Render(&GridView);
                GridStage.Render(&GridView);
                // Open the Menu
                Menu.Open(&GridView);
                break;
            case VK_LEFT:
                if (MyDroid) MyDroid->ChangeDirection( 0.05 );
                break;
            case VK_RIGHT:
                if (MyDroid) MyDroid->ChangeDirection( -0.05 );
                break;
            case VK_UP:
                if (MyDroid) MyDroid->ChangeSpeed( 0.02 );
                break;
            case VK_DOWN:
                if (MyDroid) MyDroid->ChangeSpeed( -0.02 );
                break;
            case VK_SPACE:
                if (MyDroid) MyDroid->FireWeapon(1);
                break;
            }
        }
        break;
    case WM_KEYUP:
        switch( wParam )
        {
        case VK_LEFT:
            //GridStage.SetYSpeed( 0.0 );
            break;
        case VK_RIGHT:
            //GridStage.SetYSpeed( 0.0 );
            break;
        case VK_UP:
            //GridStage.SetXSpeed( 0.0 );
            break;
        case VK_DOWN:
            //GridStage.SetXSpeed( 0.0 );
            break;
        }
        break;
    case WM_LBUTTONDOWN:
        //GridStage.FireWeapon( GridStage.m_SelectedObject, 1 );
        //PostMessage(hWnd, WM_CLOSE, 0, 0);
        break;

    case WM_MOUSEMOVE:
        nXpos = (int) LOWORD(lParam);
        nYpos = (int) HIWORD(lParam);
        //GridView.ShowMouse( nXpos, nYpos );
        break;

    case WM_DESTROY:
        GridView.DestroyGraphicsObjects();
        PostQuitMessage( 0 );
        break;
    }

    return DefWindowProc(hWnd, message, wParam, lParam);

} /* WindowProc */



HWND doWindowInit( HINSTANCE hInstance, int nCmdShow )
{
    HWND                hwnd;
    WNDCLASS            wc;

    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = GetStockBrush(BLACK_BRUSH);
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass( &wc );
    
    /*
     * create a window
     */
    hwnd = CreateWindowEx(
        0,
        NAME,
        TITLE,
        WS_POPUP,
        0,
        0,
        GetSystemMetrics(SM_CXSCREEN),
        GetSystemMetrics(SM_CYSCREEN),
        NULL,
        NULL,
        hInstance,
        NULL );

    if( hwnd )
    {
        ShowWindow( hwnd, nCmdShow );
        UpdateWindow( hwnd );
    }

    return hwnd;
}



/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                        LPSTR lpCmdLine, int nCmdShow)
{
    MSG     msg;
    HWND    hwnd;
    int endcount = 200;

    lpCmdLine = lpCmdLine;
    hPrevInstance = hPrevInstance;

    LoadGridFile( "grid1.grd" );

    hwnd = doWindowInit( hInstance, nCmdShow );
    if( !hwnd ) {
        return FALSE;
    }

    if ( !GridView.Initialize( hwnd ) ) {
        return FALSE;
    }

    GridStage.DrawConsole(&GridView);

    while( 1 )
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( !GetMessage( &msg, NULL, 0, 0 ) )
                return msg.wParam;
            TranslateMessage(&msg); 
            DispatchMessage(&msg);
        }
        else if( GridView.IsActive() )
        {
            if (Menu.IsActive()) {
                Menu.Render();
            }
            else {
                GridStage.Process();
                if ( GridStage.m_Objects[0]->m_Team != 1 ) {
                    //GridStage.m_GameStatus = 0;
                    MyDroid = 0;
                    endcount--;
                    //if (!endcount) exit(0);
                }
                GridStage.Render(&GridView);
            }
        }
        else
        {
            // make sure we go to sleep if we have nothing else to do
            WaitMessage();
        }
    }
} /* WinMain */






int LoadGridFile( char *filename )
{
    FILE *fh;
    int x = 0;
    int k;
    int y = MAX_Y_GRID_SIZE-1;
    char buf[MAX_X_GRID_SIZE*2+100];
    char flag[MAX_X_GRID_SIZE*2+100];
    CDroid* TempDroid;
    CDoor *TempDoor, *PrevDoor, *FirstDoor;
    CGridObj *TempObj;
    int objtype, team, picnum, value, yoffs;
    float xpos, ypos, diam;

    for (x=0; x<MAX_X_GRID_SIZE; x++) {
        for (y=0; y<MAX_Y_GRID_SIZE; y++) {
            // create an AsignGridTerrain method! -TDP
            GridStage.m_Grid[((int)(x))+((int)(y))*GridStage.m_GridXSize] = 0;
        }
    }

    fh = fopen( filename, "rt" );
    if (!fh) return 1;

    y = MAX_Y_GRID_SIZE-1;

    // Skip lines until we find the grid
    do {
        fgets( buf, MAX_X_GRID_SIZE*2+99, fh);
    } while (!feof(fh) && strncmp("G>", buf, 2));

    // Read in the grid
    while (!feof(fh) && y>=0) {
        x = 0;
        while (buf[x+2] && buf[x+3] && x<MAX_X_GRID_SIZE) {
            if (buf[x+2] == '.') {
                GridStage.m_Grid[((int)(x))+((int)(y))*GridStage.m_GridXSize] = 0;
            }
            else if (buf[x+2] == 'X') {
                GridStage.m_Grid[((int)(x))+((int)(y))*GridStage.m_GridXSize] = 25;
            }
            else if (buf[x+2] == '#') {
                GridStage.m_Grid[((int)(x))+((int)(y))*GridStage.m_GridXSize] = 47;
            }
            else if (isdigit(buf[x+2])) {
                GridStage.m_Grid[((int)(x))+((int)(y))*GridStage.m_GridXSize] = buf[x+2] - '0';
            }
            x++;
        }
        fgets( buf, MAX_X_GRID_SIZE*2+99, fh);
        y--;
    }

    // Read in the objects
    while (!feof(fh) && buf[0] != '.') {
        if (!strncmp(buf, "!obj:", 5)) {
            // Read in an individual object
            switch(atoi(buf+5)) {
            case OT_TANK:
                TempDroid = new CDroid();
                sscanf( buf+5, "%d %d %f %f %d",
                    &objtype, &team, &xpos, &ypos, &picnum );
                TempDroid->m_ObjType = objtype;
                TempDroid->m_Team = team;
                TempDroid->m_XPos = (double) xpos;
                TempDroid->m_YPos = (double) ypos;
                TempDroid->m_Speed = 0.04;
                TempDroid->m_Diameter = 1.0;
                TempDroid->m_Radius = 0.5;
                TempDroid->m_PicNum = picnum;
                TempDroid->ConnectToGrid(GridStage.m_Grid, GridStage.m_GridObjs,
                    GridStage.m_Objects, &(GridStage.m_TotalGridObjs));
                if (team == 1) MyDroid = TempDroid;
                break;
            case OT_DOOR:
                k=0;
                sscanf( buf+5, "%d %d %d", &objtype, &team, &value );
                do {
                    fgets( buf, MAX_X_GRID_SIZE*2+99, fh);
                    sscanf( buf, "%f %f %f %d %s",
                        &xpos, &ypos, &diam, &picnum, flag );
                    TempDoor = new CDoor();
                    TempDoor->m_ObjType = OT_DOOR;
                    TempDoor->m_XPos = xpos;
                    TempDoor->m_YPos = ypos;
                    TempDoor->m_Diameter = diam;
                    TempDoor->m_Radius = diam / 2.0;
                    TempDoor->m_PicNum = picnum;
                    TempDoor->m_Value = value;
                    if (k==0) {
                        FirstDoor = TempDoor;
                    }
                    else {
                        TempDoor->m_NextDoor = PrevDoor;
                    }
                    PrevDoor = TempDoor;
                    TempDoor->ConnectToGrid(GridStage.m_Grid, GridStage.m_GridObjs,
                        GridStage.m_Objects, &(GridStage.m_TotalGridObjs));
                    k++;
                } while (!feof(fh) && flag[0] != '.');
                FirstDoor->m_NextDoor = TempDoor;
                break;
            case OT_KEY:
            case OT_FUEL:
            case OT_AMMO:
            case OT_ARMOR:
                TempObj = new CGridObj();
                sscanf( buf+5, "%d %d %d %f %f %f %d %d",
                    &objtype, &team, &value, &xpos, &ypos, &diam, &picnum, &yoffs );
                TempObj->m_ObjType = objtype;
                TempObj->m_Team = team;
                TempObj->m_XPos = xpos,
                TempObj->m_YPos = ypos;
                TempObj->m_YOffset = yoffs;
                TempObj->m_Diameter = diam;
                TempObj->m_Radius = diam / 2.0;
                TempObj->m_PicNum = picnum;
                TempObj->m_Value = value;
                TempObj->ConnectToGrid(GridStage.m_Grid, GridStage.m_GridObjs,
                    GridStage.m_Objects, &(GridStage.m_TotalGridObjs));
                break;
            }
        }
        fgets( buf, MAX_X_GRID_SIZE*2+99, fh);
    }

    return 0;
}


