/*------------------------------------------------------------------------
Grid.h: General constants and type definitions
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/
#ifndef GRID_H_INCLUDED
#define GRID_H_INCLUDED

#include "SDL.h"

#define MAX_TANKS         100
#define MAX_X_GRID_SIZE   100
#define MAX_Y_GRID_SIZE   100
#define MAX_PICS          100
#define MAX_OBJECTS       100
#define MAX_TILES         100
#define MAX_AUDIO_EVENTS  5
#define MAX_SOUNDS        20

#define PIC_IS_OFF_SCREEN 1

#define GS_SELECTED 1
#define GS_VISABLE  2
#define GS_SOLID    4
#define GS_DORMANT  8

#define GS_CURSOR_UP      -1000

#define DP_FULL           0
#define DP_LEFT           1
#define DP_RIGHT          2
#define DP_CONSOLE        3
#define DP_HILIGHT        4
#define DP_OUTLINE        5
#define DP_SHADOW         6

#define OT_TANK           1
#define OT_FIREBALL       2
#define OT_MISSILE        3
#define OT_SMOKE          4
#define OT_EXPLOSION      5
#define OT_DOOR           6
#define OT_KEY            7
#define OT_FUEL           8
#define OT_AMMO           9
#define OT_ARMOR          10

#define TWO_PI_RADIANS    6.280000

typedef struct {
	SDL_Surface *image;
    int left;
    int right;
    int top;
    int bottom;
    int height;
    //int xoffset;
    //int yoffset;
    int frames;
    int currentframe;
    int delay;
} GPic;


typedef struct {
    Uint32 len;
    Uint8 *buf;
} GSound;


typedef struct {
    Uint32 len;
    Uint8 *pos;
} GAudioEvent;


typedef struct {
    int* Grid;
    void* GridObjs;
    void* Tiles;
    void* Objects;
    int* TotalGridObjs;
    int* TotalGridTiles;
    GAudioEvent *AudioQueue;
    GSound *Sounds;
} GGridInfo;


#endif //#ifndef GRID_H_INCLUDED
