/*------------------------------------------------------------------------
GridObj.h: interface for the CGridObj class.
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/

#if !defined(GRIDOBJ_H_INCLUDED)
#define GRIDOBJ_H_INCLUDED

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <math.h>
#include "GridView.h"
#include "GridTile.h"

#define GRID(x,y)      m_Grid[((int)(x))+((int)(y))*m_GridXSize]
#define TILE(x,y)      m_Tiles[m_Grid[((int)(x))+((int)(y))*m_GridXSize]]
#define GRIDOBJS(x,y)  m_GridObjs[((int)(x))+((int)(y))*m_GridXSize]


class CGridObj  
{
public:
    CGridObj();

    CGridObj(GGridInfo *Info);
    void ConnectToGrid(GGridInfo *Info);
    void Initialize( void );
    virtual ~CGridObj();
    virtual int Render(CGridView *gv, unsigned long lClockTicks,
        int nScreenX, int nScreenY);
    virtual int Process(unsigned long lClockTicks, double dTimeSlice);
    void AddObjToSquare( void );
    void RemoveObjFromSquare( void );
    void DestroyObject( void );
    int CollisionCheck( double nXpos, double nYpos, CGridObj **pObj );
    CGridObj* ObjCollisionCheck( double nXpos, double nYpos, double nXcorner, double nYcorner );
    int PlaySound( int SoundNum );
    GGridInfo* FillGridInfo(GGridInfo *Info);
    inline int IsSelected(void) { return m_Status & GS_SELECTED; };
    inline void Select(void) { m_Status |= GS_SELECTED; };
    inline void UnSelect(void) { m_Status &= (0xffff - GS_SELECTED); };

    int    m_ObjType;
    int    m_ObjNum;
    double m_XPos;
    double m_YPos;
    int    m_XOffset;
    int    m_YOffset;
    double m_Diameter;
    double m_Radius;
    //double m_RadiusSquared;
    int    m_PicNum;
    int    m_Value;
    int    m_Solid;
    unsigned int m_Status;
    long   m_Team;

    int m_GridXSize;
    int* m_Grid;
    int* m_pTotalGridObjs;
    CGridTile** m_Tiles;
    CGridObj** m_GridObjs;
    CGridObj** m_Objects;
    GAudioEvent* m_AudioQueue;
    GSound*      m_Sounds;
    CGridObj* m_NextObj;
    CGridObj* m_PrevObj;
};

#endif // !defined(GRIDOBJ_H_INCLUDED)
