/*------------------------------------------------------------------------
GridStage.cpp: implementation of the CGridStage class
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:  http://www.gridslammer.org
                            info@gridslammer.org

------------------------------------------------------------------------*/
#include "GridStage.h"
#include "GridActor.h"
#include <stdio.h>
#ifdef TARGET_LINUX
#include <string.h>
#include <ctype.h>
#endif



//-------------------------------------------------------------------------
// Method:    CGridStage
// Paramters: none
//
// Returns:   none
//
// This is the constructor for the CGridStage class.
//-------------------------------------------------------------------------
CGridStage::CGridStage()
{
    int k;

    for (k=0; k<MAX_OBJECTS; k++) m_Objects[k] = NULL;

    m_SelectedObject = 0;

	// Most of this stuff should really be initialized
	// from a config file -TDP

    m_GridXSize = MAX_X_GRID_SIZE;
    m_GridYSize = MAX_Y_GRID_SIZE;

    m_xpos = 50.5;
    m_ypos = 50.5;

    m_xspeed = 0.0;
    m_yspeed = 0.0;

    m_screenx = 300;
    m_screeny = 100;
    m_dir  = 1;
    m_diff = 0.25;

    m_MouseX = 0;
    m_MouseY = 0; 

    m_NorthXFactor = 23;
    m_NorthYFactor = -14;

    m_EastXFactor  = -26;
    m_EastYFactor  = -12;

    m_XBaseShift = -26;
    m_YBaseShift = 0;

    // these are duplicated in CGridView... need to fix that.  -TDP
    m_ViewTopBorder    = 0;
    m_ViewBottomBorder = 401;
    m_ViewLeftBorder   = 0;
    m_ViewRightBorder  = 640;

    m_TotalGridObjs = 0;
    m_TotalGridTiles = 0;

    m_StartXOffset = 2;
    m_StartYOffset = 5;

    m_Ticks = 0;
    m_SliceTicks = 30;
    m_InfiniteWallTile = 25;  // set this to -1 to turn off infinite walls
}



//-------------------------------------------------------------------------
// Method:    ~CGridStage
// Paramters: none
//
// Returns:   none
//
// This is the destructor for the CGridStage class.
//-------------------------------------------------------------------------
CGridStage::~CGridStage()
{
}




//-------------------------------------------------------------------------
// Method: Render
// Parameters: gv = A pointer to the games CGridView object
//
// This method is called to render a single frame of the game.  Override
// this function to customize how your game appears, such as to add
// indicators or status bars to the game interface.  The general sequence
// of events in this function is usually to clear the background, call
// the RenderGrid method, draw any UI elements that should overlay the
// game view, and then call the gv->FlipSurface() method.
//-------------------------------------------------------------------------
int CGridStage::Render( CGridView *gv )
{
    gv->SetTicks(m_Ticks);  // this is a kludge!  -TDP

    // Draw the game background
    DrawBackground(gv);

    // set the perspective to the first object in the list
    if (m_Objects[0]->m_Team == 1) {
        m_xpos = m_Objects[0]->m_XPos;
        m_ypos = m_Objects[0]->m_YPos;
    }

    // Call the method that renders the map and game characters
    RenderGrid( gv, m_xpos + 13.0, m_ypos - 0.5);

    // flip the back buffer video page to the active video page.
    gv->FlipSurface();

    return 0;
}




//-------------------------------------------------------------------------
// Method:    DrawBackground
// Paramters: gv = Pointer to the gridview to draw into.
//
// Returns:   none
//
// This routine is called by the framework before each frame is
// rendered.  By default it simply clears the entire view area.  Override
// it with your own function if you want to draw a more interesting
// background.
//-------------------------------------------------------------------------
void CGridStage::DrawBackground( CGridView *gv )
{
    gv->ClearBackground();
}




//-------------------------------------------------------------------------
// Method:    DrawConsole
// Paramters: gv = Pointer to the gridview to draw into.
//
// Returns:   none
//
// This routine is called by the framework after each frame is
// rendered.  It draws any console widgets (score keeper, health
// indicator, etc) onto the screen.  The provided version does nothing,
// so override it with your own implementation.
//-------------------------------------------------------------------------
void CGridStage::DrawConsole( CGridView *gv )
{
}



//-------------------------------------------------------------------------
// Method:    RenderGrid
// Paramters: gv = A pointer to the CGridView to render into
//            nXpos, nYpos = the x and y grid coordinates of map point
//               that should appear in the upper left corner of the
//               grid view.
//
// This function draws the current state of the game map and characters
// into the provided grid view.
//-------------------------------------------------------------------------
void CGridStage::RenderGrid( CGridView *gv, double nXpos, double nYpos )
{
    int x, y;                // X/Y grid coordinates of the grid tile to render
    int xstart, ystart;      // The first tile to render
    int j,k;                 // Loop counters
    int nXdiff, nYdiff;      // Used to calculate tile screen positions
    int xpos, ypos;          // The screen position to render a tile into
    int nGridX, nGridY;      // The tile that appears in the extreme upper left
    int nScreenX, nScreenY;  // The screen position of the upper left tile
    int result;              // Return value from image drawing operations
    CGridObj *gobj;          // Pointer to the cursor selected grid object

    // This is the grid square that is currently in the upper right corner.
    nGridX = (int) nXpos;
    nGridY = (int) nYpos;

    // Find the lower left corner where we will start to draw that square.
    nScreenX = m_ViewRightBorder - 
               (int) ( ((double) m_NorthXFactor * (nXpos - (double) nGridX))  +
                       ((double) m_EastXFactor * (nYpos - (double) nGridY)) ) +
                     m_XBaseShift;
    nScreenY = m_ViewTopBorder -
               (int) ( ((double) m_NorthYFactor * (nXpos - (double) nGridX)) +
                       ((double) m_EastYFactor * (nYpos - (double) nGridY))) +
                     m_YBaseShift;

    // calculate an offset starting position to render partially offscreen
    // tiles.
    xstart = nGridX+m_StartXOffset + 3;
    ystart = nGridY+m_StartYOffset - 3;

    // Get the current mouse position
    gv->GetMousePos( &m_MouseX, &m_MouseY );

    // Calculate the position within the grid that the mouse is
    // currently pointing to and set the tile cursor to that position
    m_CursorX = nXpos + ((double) (m_ViewRightBorder - m_MouseX-15)) /
              ((double) m_EastXFactor * 2 )
              + ((double) m_MouseY-60) / ((double) m_EastYFactor * 2);
    m_CursorY = nYpos + ((double) (m_ViewRightBorder - m_MouseX-15)) /
              ((double) m_NorthXFactor * 2)
              + ((double) m_MouseY-60) / ((double) m_NorthYFactor * 2);

    // If an object is near the tile cursor, mark it as selected.
    if ( gobj = GetClosestObject(m_CursorX, m_CursorY) ) {
        gobj->Select();
    }

    // Now loop through horizontal rows of tiles.  Because the isometric
    // view of the grid is 'at an angle', we will actually be traversing
    // tiles diagonally through the grid for each horizontal stripe.
    // We will iterate each stripe three times: first for the floor tiles,
    // a second time for mobile objects, and a third time for walls and
    // other 'tall' tiles.
    // This should eventually be changed to a one pass technique that
    // orders the tile and object rendering by height.
    for (j=0; j<38; j++) {

        // Render the floor tiles in this horizontal stripe
        for (k=0; k<20; k++) {
            // Calculate the x/y coordinates of the tile to render
            x = xstart - k;
            y = ystart + k;

            // Is that tile actually in the grid?
            if (x<MAX_X_GRID_SIZE && x >= 0 &&
                y<MAX_Y_GRID_SIZE && y >= 0 ) {

                // Calculate the distance (measured in tiles) of the current
                // tile from the upper leftmost tile.
                nXdiff = x - nGridX;
                nYdiff = y - nGridY;

                // Calculate the screen position to draw the current tile at
                xpos = nScreenX + m_NorthXFactor*nXdiff + m_EastXFactor*nYdiff;
                ypos = nScreenY + m_NorthYFactor*nXdiff + m_EastYFactor*nYdiff;

                // Is the current tile a floor tile?
                if ( !m_Tiles[GRID(x, y)]->m_Solid ) {

                    // Draw the tile
                   m_Tiles[GRID(x,y)]->Render( gv, m_Ticks, xpos, ypos );

                    // Is the tile cursor on the current tile?
                    if ( x == (int) m_CursorX && y == (int) m_CursorY ) {
                        // Draw the tile cursor
                        //gv->DrawPic( 48, xpos, ypos -
                        //    gv->m_Pics[48].height + 53 );
                        // Draw the small mouse position cursor
                        //gv->DrawPic( 50, m_MouseX + 5, m_MouseY - 6 );
                        m_Tiles[GRID(x,y)]->DrawCursor(gv, m_Ticks,
                            xpos, ypos, m_MouseX, m_MouseY);
                    }
                }
            }
        }
 
        // Render all the mobile objects in this horizontal stripe
        for (k=0; k<20; k++) {

            // Calculate the x/y coordinates of the tile to render
            x = xstart - k;
            y = ystart + k;

            // Is that tile actually in the grid?
            if (x<MAX_X_GRID_SIZE && x >= 0 &&
                y<MAX_Y_GRID_SIZE && y >= 0 ) {

                // Calculate the distance (measured in tiles) of the current
                // tile from the upper leftmost tile.
                nXdiff = x - nGridX;
                nYdiff = y - nGridY;

                // Calculate the screen position to draw the current tile at
 
                xpos = nScreenX + m_NorthXFactor*nXdiff + m_EastXFactor*nYdiff;
                ypos = nScreenY + m_NorthYFactor*nXdiff + m_EastYFactor*nYdiff;

                // Draw any mobile objects in this square 
                if ( GRIDOBJS(x,y) ) {
                    RenderGridSquare( gv, x, y, nScreenX, nScreenY,
                         nGridX, nGridY);
                }
            }
        }

        // Render all tall tiles in this horizontal stripe
        for (k=0; k<20; k++) {

            // Calculate the x/y coordinates of the tile to render
            x = xstart - k;
            y = ystart + k;

            // Is that tile actually in the grid?
            if (x<MAX_X_GRID_SIZE && x >= 0 &&
                y<MAX_Y_GRID_SIZE && y >= 0 ) {

                // Calculate the distance (measured in tiles) of the current
                // tile from the upper leftmost tile.
                nXdiff = x - nGridX;
                nYdiff = y - nGridY;

                // Calculate the screen position to draw the current tile at
                xpos = nScreenX + m_NorthXFactor*nXdiff + m_EastXFactor*nYdiff;
                ypos = nScreenY + m_NorthYFactor*nXdiff + m_EastYFactor*nYdiff;

                // Is the current tile a wall tile?
                if ( m_Tiles[GRID(x, y)]->m_Solid ) {

                    // Draw the tile
                   m_Tiles[GRID(x,y)]->Render( gv, m_Ticks, xpos, ypos );

                    // Is the tile cursor in this tile?
                    if ( x == (int) m_CursorX && y == (int) m_CursorY ) {
                        // Draw the tile cursor
                        //gv->DrawPic( 48, xpos, ypos -
                        //     gv->m_Pics[GRID(x,y)].height + 53 );
                        // Draw the mouse position indicator
                        //gv->DrawPic( 50, m_MouseX+5, m_MouseY - 
                        //     gv->m_Pics[GRID(x,y)].height +
                        //     gv->m_Pics[48].height - 6 );
                        m_Tiles[GRID(x,y)]->DrawCursor(gv, m_Ticks,
                            xpos, ypos, m_MouseX, m_MouseY);
                    }
                }
            }
            else if ( m_InfiniteWallTile >= 0 ) {
                // Draw infinite walls extending away from the edges
                // of the map
                nXdiff = x - nGridX;
                nYdiff = y - nGridY;
                xpos = nScreenX + m_NorthXFactor*nXdiff + m_EastXFactor*nYdiff;
                ypos = nScreenY + m_NorthYFactor*nXdiff + m_EastYFactor*nYdiff;

                result = gv->DrawPic( m_InfiniteWallTile, xpos, ypos
                          - gv->m_Pics[m_InfiniteWallTile].height + 53);
            }
        }

        // Choose the starting tile for the next horizontal stripe
        // of tiles to be render.
        if (j%2) {        // On odd counts of j,
            ystart--;     // decrement the y starting point for the horizontal
        }                 //    stripe of tiles.
        else {
            xstart--;     // On even counts of j,
        }                 // decerement the x starting point for the horizontal
    }                     //    stripe of tiles.

    return;
}




//-------------------------------------------------------------------------
// Method:   RenderGridSquare
// Paramters: gv = A pointer to the GridView object to render into
//            x, y = The coordinates of the tile containing the objects to render
//            nScreenX, nScreenY = The screen position to render at
//            nGridX, nGridY = The coordinates of the tile in the upper left
//                             corner of the screen.
//
// Returns:   zero on success, non zero on error.
//
// This function renders all of the grid objects located in a given tile.
//-------------------------------------------------------------------------
int CGridStage::RenderGridSquare( CGridView *gv, int x, int y, int nScreenX, int nScreenY,
                               int nGridX, int nGridY )
{
    double dXdiff;
    double dYdiff;
    int xpos;
    int ypos;
    int result;
    CGridObj* gobj = GRIDOBJS(x, y);

	// Loop through all of the objects in this grid square
    while (gobj) {
		// Calculate the object's distance from the upper
		// right corner of the screen
        dXdiff = gobj->m_XPos - (double) nGridX;
        dYdiff = gobj->m_YPos - (double) nGridY;

		// calculate the screen position at which which to render
		// the object
        xpos = nScreenX + (int) (((double) m_NorthXFactor * dXdiff ) +
                                 ((double) m_EastXFactor * dYdiff ));
        ypos = nScreenY + (int) (((double) m_NorthYFactor * dXdiff ) +
                                 ((double) m_EastYFactor * dYdiff ));

		// Render the object
        result = gobj->Render( gv, m_Ticks, xpos, ypos);
        gobj->UnSelect();   // Kludge Alert!  -TDP

		// Get the next object in the linked list
        gobj = gobj->m_NextObj;
    }

    return 0;
}





//-------------------------------------------------------------------------
// Method:    LoadGrid
// Paramters: filename = the name of the file containing the grid map
//                        and game state data.
//
// Returns:   zero on sucess, non zero on failure
//
// Currently there is no default implementation for LoadGrid, so override
// this with your own code.  Look at the DroidWars source code for an
// example.
//-------------------------------------------------------------------------
int CGridStage::LoadGrid( char *filename )
{
    return 0;
}





//-------------------------------------------------------------------------
// Method:    DebugMsg
// Paramters: msg = Pointer to a message string to write to a log file.
//
// Returns:   none
//
// This is just here to help with debugging.
//-------------------------------------------------------------------------
void CGridStage::DebugMsg( char *msg )
{
    FILE *fh;

    fh = fopen( "c:\\lcdebug.log", "a+" );
    fprintf( fh, "%s", msg );
    fclose(fh);

    return;
}




//-------------------------------------------------------------------------
// Method:    Process
// Paramters: none
// Returns:   zero on success, non zero on error.
//
// This function iterates throug the list of all grid objects, calling
// the Process method on each one.  This is essentially what drives the
// entire game sim.
//-------------------------------------------------------------------------
int CGridStage::Process( void )
{
    int j, k;               // loop counters
    int loops;              // the number of process iterations to do
    double timeslice;       // the amount of game sim time to process
    unsigned long newticks; // temporarily hold the current clock tick
    unsigned long elapsed;  // the elapsed clock ticks since the beginning
                            //   of the previous Process call

    newticks = (unsigned long) SDL_GetTicks();  // Get the current clock
    elapsed = newticks - m_Ticks;            // Find time of previous iteration
    m_Ticks = newticks;                      // Store the current clock

    // Find out the number of Process iterations to do
    loops = elapsed / m_SliceTicks + 1;
    // determine the time slice to process
    timeslice = ((double) elapsed / (double) m_SliceTicks) / loops;

    //printf( "loops=%d  timeslice=%f newticks=%ld\n", loops,
    //       (float) timeslice, newticks);

    // loop through each iteration
    for (j=0; j<loops; j++) {
        k = 0;  // start at the first grid object in the active list
        // loop through all active objects
        do {
            // call the Process method on each object
            m_Objects[k++]->Process(m_Ticks, timeslice);
        } while ( k < m_TotalGridObjs );
    }

    return 0;  // return with no errors
}




//-------------------------------------------------------------------------
// Method:    GetGridObject
// Paramters: nObjNum = The number of the grid object to get.
//
// Returns:   A pointer to the actual instantiated grid object
//            corresponding to the object number supplied.
//
// Use this number to retrieve a pointer to an actual grid object based on
// its unique object number.  Currently that number is just a index into
// an array, but that could change in some future implementation, so use
// this function instead of accessing the array directly.
//-------------------------------------------------------------------------
CGridObj* CGridStage::GetGridObject( int nObjNum )
{
    return m_Objects[nObjNum];
}




//-------------------------------------------------------------------------
// Method:   GetClosestObject
// Paramters: xpos, ypos = the grid coordinates to check for an object.
// Returns:   A pointer to grid object, or NULL if no object is near.
//
// This function finds the closest object within a half grid unit of the
// given coordinates.  It is used primary by the grid cursor code to
// highlight and select objects.
//-------------------------------------------------------------------------
CGridObj* CGridStage::GetClosestObject(double xpos, double ypos)
{
    int x;
    int y;
    int k;
    double xd, yd;
    CGridObj* gobj;

    // This entire thing is an ugly hack that should be fixed. -TDP
    for (k=0; k<4; k++) {
        switch(k) {
            case 0: x = (int) xpos; y = (int) ypos; break;
            case 1: x = (int) xpos-1; y = (int) ypos; break;
            case 2: x = (int) xpos; y = (int) ypos-1; break;
            case 3: x = (int) xpos-1; y = (int) ypos-1; break;
        }
        if (x<0 || x>=MAX_X_GRID_SIZE || y<0 || y>=MAX_Y_GRID_SIZE) 
            continue;
        gobj = GRIDOBJS(x, y);
        while (gobj) {
            xd = xpos - (gobj->m_XPos + gobj->m_Radius);
            yd = ypos - (gobj->m_YPos + gobj->m_Radius);
            if (xd*xd + yd*yd < 0.25) return gobj;
            gobj = gobj->m_NextObj;
        }
    }

    return 0;
}




//-------------------------------------------------------------------------
// Method:    GetGridTile
// Paramters: x, y = the x and y grid coordinates of the tile
//
// Returns:   The integer value corresponding to the tile type
//
// This function returns the unique integer value corresponding to the
// tile type occupying the given grid position.  Use this function rather
// than manipulating the tile array directly.
//-------------------------------------------------------------------------
int CGridStage::GetGridTile(int x, int y)
{
    return GRID(x, y);
}



//-------------------------------------------------------------------------
// Method:    SetGridTile
// Paramters: x, y = the x and y grid coordinates of the tile
//            tile = the integer value of the tile type to assign
//
// Returns:   The type of the tile it is replacing.
//
// This function set the tile type at the given grid position.
// Use this function rather than manipulating the tile array directly.
//-------------------------------------------------------------------------
int CGridStage::SetGridTile(int x, int y, int tile)
{
    int oldtile = GRID(x,y);
    GRID(x,y) = tile;
    return oldtile;
}



//-------------------------------------------------------------------------
// Method:    LoadTiles
// Paramters: filename = the name of the file containing the data
//                       for initializing tile classes.
//
// Returns:   zero on sucess, non zero on failure
//
// Loads the tile data and initializes the array of CGridTile classes.
//-------------------------------------------------------------------------
int CGridStage::LoadTiles( CGridView *gv, char *filename )
{
    // This is all a temporary hack! -TDP
    int k = 0;

    while ( k < 60 ) {
        m_Tiles[k] = new CGridTile;
        m_Tiles[k]->m_PicNum = k;
        m_Tiles[k]->m_Height = gv->m_Pics[k].height;
        m_Tiles[k]->m_Solid = ( k > 3 );
        k++;
    }

    m_TotalGridTiles = k;
    return m_TotalGridTiles;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::GetCursorState()
{
	return m_CursorState;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::GetCursorPosition(int* x, int* y)
{
	return 0;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::GetSelectedObjects(CGridObj** objlist, int maxobjs)
{
	return 0;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::SetCursorState(int state)
{
	int oldstate = m_CursorState;
	m_CursorState = state;
	return oldstate;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::SetCursorPosition(int x, int y)
{
	return 0;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::SetSelectMask(int mask)
{
	return 0;
}




//-------------------------------------------------------------------------
// Method:    
// Paramters: 
//
// Returns:   zero on sucess, non zero on failure
//
//-------------------------------------------------------------------------
int CGridStage::SetSmartCursor(int flag)
{
	return 0;
}



