/*------------------------------------------------------------------------
GridStage.h: interface for the CGridStage class.
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/

#if !defined(STAGE_H_INCLUDED)
#define STAGE_H_INCLUDED

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "GridTile.h"
#include "GridObj.h"
#include "GridView.h"
#include "math.h"

class CGridStage  
{
public:
    CGridStage();
    virtual ~CGridStage();
    int  Render( CGridView *gv );
    void DrawConsole( CGridView *gv );
    void DrawBackground( CGridView *gv );
    void RenderGrid( CGridView *gv, double nXpos, double nYpos  );
    int  RenderGridSquare( CGridView *gv, int x, int y, int nScreenX, int nScreenY,
                               int nGridX, int nGridY);
    int  LoadGrid( char *filename );
    void DebugMsg( char *msg );
    int  Process( void );
    CGridObj* GetGridObject( int nObjNum );
    CGridObj* GetClosestObject( double x, double y );
    int GetGridTile(int x, int y);
    int SetGridTile(int x, int y, int tile);
    int LoadTiles( CGridView *gv, char *filename );

	int GetCursorState();
	int GetCursorPosition(int* x, int* y);
	int GetSelectedObjects(CGridObj** objlist, int maxobjs);

	int SetCursorState(int state);
	int SetCursorPosition(int x, int y);
	int SetSelectMask(int mask);
	int SetSmartCursor(int flag);

    int m_SelectedObject;
    CGridView  m_GridView;
    CGridTile* m_Tiles[MAX_TILES];
    CGridObj*  m_Objects[MAX_OBJECTS];
    CGridObj*  m_GridObjs[MAX_X_GRID_SIZE * MAX_Y_GRID_SIZE];
    int        m_Grid[MAX_X_GRID_SIZE * MAX_Y_GRID_SIZE];
    int m_GridXSize;
    int m_GridYSize;
    int m_TotalGridObjs;
    int m_TotalGridTiles;
    int m_InfiniteWallTile;

protected:

    double   m_xpos;
    double   m_ypos;
    double   m_xspeed;
    double   m_yspeed;
    int      m_screenx;
    int      m_screeny;
    int      m_dir;
    double   m_diff;

    int m_NorthXFactor;
    int m_EastXFactor;
    int m_NorthYFactor;
    int m_EastYFactor;

    int m_XBaseShift;
    int m_YBaseShift;

    int m_ViewTopBorder;
    int m_ViewBottomBorder;
    int m_ViewLeftBorder;
    int m_ViewRightBorder;

    int m_StartXOffset;
    int m_StartYOffset;

    unsigned long m_Ticks;
    unsigned int m_SliceTicks;
    int m_MouseX;
    int m_MouseY;
public:
    double m_CursorX;
    double m_CursorY;
    int m_CursorState;
};

#endif // !defined(STAGE_H_INCLUDED)
