/*------------------------------------------------------------------------
GridTile.cpp: Implementation of the CGridTile class.
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:  http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/
#include "GridTile.h"



//-------------------------------------------------------------------------
// Method:    CGridTile
// Paramters: none
//
// Returns:   nothing
//
// The constructor.
//-------------------------------------------------------------------------
CGridTile::CGridTile()
{
    m_TileCursorPic = 48;
    m_MouseCursorPic = 50;
    m_XCursorOffset = 5;
    m_YCursorOffset = -6;
}



//-------------------------------------------------------------------------
// Method:    CGridTile
// Paramters: Info = pointer to a a CGridInfo structure
//
// Returns:   nothing
//
// This version of the constructor instantiates the grid object and then
// connects it to the game grid by calling the ConnectToGrid method.
//-------------------------------------------------------------------------
CGridTile::CGridTile(GGridInfo *Info)
{
}



//-------------------------------------------------------------------------
// Method:    ~CGridTile
// Paramters: none
//
// Returns:   nothing
//
// The destructor.
//-------------------------------------------------------------------------
CGridTile::~CGridTile()
{
}



//-------------------------------------------------------------------------
// Method:    Render
// Paramters: gv = pointer to the GridView to render into
//            lClockTicks = the number of milliseconds since game start
//            ScreenX, nScreenY = the screen coordinates to draw to
//
// Returns:   Zero on success, non zero on failure
//
// Draws the grid tile to the screen at the supplied coordinates.
// This function is called by the framework as part of the overall
// rendering of the game.
//
// Override this method with your own code if you want to do something
// other than draw a static image.
//-------------------------------------------------------------------------
int CGridTile::Render(CGridView *gv, unsigned long lClockTicks,
        int nScreenX, int nScreenY)
{
    gv->DrawPic( m_PicNum, nScreenX, nScreenY - m_Height + 53 );
    return 0;
}




//-------------------------------------------------------------------------
// Method:    CollisionCheck
// Paramters: nXpos, nYpos = the position within the tile to check for
//                           a collision
//
// Returns:   Zero for no collision, non zero for collision
//
// This function takes a point within a tile and returns 1 of it collides
// with the tile and 0 if not.  You should override this function if you
// want to create a tile that is only partially blocked.  When examining
// the nXpos or nYpos coordinates, be sure to discard the whole number
// (left side of the decimal) portion.
//-------------------------------------------------------------------------
int CGridTile::CollisionCheck( double nXpos, double nYpos )
{
    return m_Solid;
}



//-------------------------------------------------------------------------
// Method:    DrawCursor
// Paramters: gv = pointer to the GridView to render into
//            lClockTicks = the number of milliseconds since game start
//            ScreenX, nScreenY = the screen coordinates to draw the tile
//                                cursor to
//            nMouseX, nMouseY = the screen coordinates to draw the mouse
//                               cursor to
//
// Returns:   Zero on success, non zero on failure
//
// This function draws the tile and mouse cursors in the appropriate
// manner for the tile.  Override this function if you want the cursor
// to change as it traverses different types of terrain.
//-------------------------------------------------------------------------
int CGridTile::DrawCursor(CGridView *gv, unsigned long lClockTicks,
        int nScreenX, int nScreenY, int nMouseX, int nMouseY)
{
    // Draw the tile cursor
    gv->DrawPic( m_TileCursorPic, nScreenX, nScreenY - m_Height + 53 );
    // Draw the small mouse position cursor
    gv->DrawPic( m_MouseCursorPic, nMouseX + m_XCursorOffset, 
        nMouseY + m_YCursorOffset - m_Height + 28);

    return 0;
}


