/*------------------------------------------------------------------------
GridView.h: interface for the CGridView class.
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:    http://www.gridslammer.org
                            info@gridslammer.org

This class exists to abstract away the actual drawing code so the
whole mess will be more portable.
------------------------------------------------------------------------*/

#ifndef GRIDVIEW_H_INCLUDED
#define GRIDVIEW_H_INCLUDED

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Grid.h"

#include "SDL.h"
#include "SDL_timer.h"


class CGridView  
{
public:
    CGridView();
    virtual ~CGridView();
    void FlipSurface( void );
    inline int DrawConsolePic(int nPicNum, int nScreenX, int nScreenY)
        { return DrawPic(nPicNum, nScreenX, nScreenY, DP_CONSOLE); };
    int DrawPic(int nPicNum, int nScreenX, int nScreenY)
        { return DrawPic(nPicNum, nScreenX, nScreenY, DP_FULL); };
    int  DrawPic( int nPicNum, int nScreenX, int nScreenY, int nDrawType );
    int  TranPic8( SDL_Rect *srcrect, SDL_Rect *dstrect, int nDrawType );
    int  TranPic16( SDL_Rect *srcrect, SDL_Rect *dstrect, int nDrawType );
    int Initialize( void );
    int InputPending( void );
    int GetInput( void );
    void GetMousePos( int *xpos, int *ypos );
    void SetActive( int bParam );
    int IsActive();
    void ClearBackground();
    void ClearBox(int left, int top, int right, int bottom);
    void DrawString( int xpos, int ypos, char *str );
    void DrawLine( int x1, int y1, int x2, int y2);
    int  ShowMouse( int nXpos, int nYpos );
    inline void SetTicks(unsigned long Ticks) { m_Ticks = Ticks; };
    int LoadPics( char *filename );
    int LoadSounds( char *filename );
    GPic* GetPic( int PicNum );
    int PlaySound( int SoundNum );

    SDL_Surface *m_image;
    SDL_Surface *m_imagebmp;
    SDL_Surface *m_screen;
    SDL_Surface *m_locked;
    SDL_AudioSpec m_AudioSpec;

    char m_strBitmap[256];
    int  m_Active;        // is application active?
    GPic m_Pics[MAX_PICS];
	GPic m_PicAreas[MAX_PICS];
    int  m_TotalPics;
    Uint8 m_ShadowColors[256];

    GAudioEvent m_AudioQueue[MAX_AUDIO_EVENTS];
    GSound      m_Sounds[MAX_SOUNDS];
    int         m_TotalSounds;

    int m_ViewTopBorder;
    int m_ViewBottomBorder;
    int m_ViewLeftBorder;
    int m_ViewRightBorder;

    unsigned long m_Ticks;
    int m_MouseX;
    int m_MouseY;
};


extern void fill_audio(void *udata, Uint8 *stream, int len);

#endif // GRIDVIEW_H_INCLUDED
