/*------------------------------------------------------------------------
GGIGridView.cpp: GGI implementation of the CGridView class.
GridSlammer Game Engine - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Examine the license.txt file located in the GridSlammer directory
    to see the full text of the LGPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:  http://www.gridslammer.org
                            info@gridslammer.org

------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "GridView.h"
#include "SDL.h"
#include "SDL_timer.h"
#include "SDL_events.h"
#include "SDL_audio.h"
#include "SDL_error.h"
 



typedef struct {
    int top;
    int bottom;
    int left;
    int right;
} RECT;




CGridView::CGridView()
{
    strcpy(m_strBitmap, "images.bmp");

    m_ViewTopBorder    = 0;
    m_ViewBottomBorder = 401;
    m_ViewLeftBorder   = 0;
    m_ViewRightBorder  = 640;

    //m_ViewTopBorder    = 25;
    //m_ViewBottomBorder = 375;
    //m_ViewLeftBorder   = 25;
    //m_ViewRightBorder  = 615;

    m_MouseX = 0;
    m_MouseY = 0;

    LoadPics("pics.dat");  // This should happen after Initialize -TDP
}



CGridView::~CGridView()
{
    // We should free video resources here! -TDP
    // Need to check that the audio queue is empty! -TDP
    SDL_CloseAudio();
}





void CGridView::FlipSurface( void )
{
    //SDL_UpdateRect(m_screen, 0,0,0,0);
    SDL_Flip(m_screen);
}








//-------------------------------------------------------------------------
// Method:    DrawPic
// Paramters: nPicNum = the number of the GPic to draw
//            nScreenX, nScreenY = screen coordinates to draw the pic at
//            nDrawType = type of draw opperation to do (straight blit,
//                        shadow effect, highlight, ...
// Returns:   0 on success, 1 on failure
//
// This function blits an image from the offscreen cache to the screen
// with optional special effects.
// NOTE: This code has been translated from DirectX to GGI to SDL... and
// from straight C to C++ as well, so it is ugly and in much need of
// cleaning up and optimizing. -TDP
//-------------------------------------------------------------------------
int CGridView::DrawPic(int nPicNum, int nScreenX, int nScreenY, int nDrawType)
{
    RECT      rect;
    int xpos;
    int ypos;
    int width;
    int height;
    int screenx;
    int left;
    int right;
    int bottom;
    int top;
    int ViewTopBorder;
    int ViewBottomBorder;
    int ViewLeftBorder;
    int ViewRightBorder;
    SDL_Rect srcrect, dstrect;

	// Get horizontal coordinates for the picture in the image cache
    screenx = nScreenX;
    width = m_Pics[nPicNum].right - m_Pics[nPicNum].left;
    left = m_Pics[nPicNum].left + width*m_Pics[nPicNum].currentframe;
    right = left + width;

    if (nDrawType == DP_CONSOLE) {
		// Use the entire screen as a the drawing surface
		// Note: this numbers should not be hardcoded like this. -TDP
        ViewTopBorder    = 0;
        ViewBottomBorder = 480;
        ViewLeftBorder   = 0;
        ViewRightBorder  = 640;
    }
    else {
		// Restrict the blit to only the active game window
		// (i.e. where the monsters and map and such are being drawn)
        ViewTopBorder    = m_ViewTopBorder;
        ViewBottomBorder = m_ViewBottomBorder;
        ViewLeftBorder   = m_ViewLeftBorder;
        ViewRightBorder  = m_ViewRightBorder;
    }

	// Get verticle coordinates for the picture in the image cache
    top = m_Pics[nPicNum].top;
    bottom = m_Pics[nPicNum].bottom;
    height = bottom - top;

	// Check if the image is completely off screen
    if ( screenx > ViewRightBorder
        || (screenx + width) < ViewLeftBorder
        || nScreenY > ViewBottomBorder
        || (nScreenY + height) < ViewTopBorder )
        return PIC_IS_OFF_SCREEN;

	// Make sure the image is not extending off the left or right border
    // Crop the coordinates if necessary
    if (screenx < ViewLeftBorder) {
        xpos = ViewLeftBorder;
        rect.left = left + xpos - screenx;
        rect.right = right;
    }
    else if ( (screenx + width) > ViewRightBorder ) {
        xpos = screenx;
        rect.left = left;
        rect.right = left + ViewRightBorder - xpos;
    }
    else {
        xpos = screenx;
        rect.left = left;
        rect.right = right;
    }

	// Make sure the image is not extending off the top or bottom border
    // Crop the coordinates if necessary
    if (nScreenY < ViewTopBorder) {
        ypos = ViewTopBorder;
        rect.top = top + ViewTopBorder - nScreenY;
        rect.bottom = bottom;
    }
    else if ( (nScreenY + height) > ViewBottomBorder ) {
        ypos = nScreenY;
        rect.top = top;
        rect.bottom = top + ViewBottomBorder - ypos;
    }
    else {
        ypos = nScreenY;
        rect.top = top;
        rect.bottom = bottom;
    }

	// Handle animated images.  This should probably be dropped from
	// GPic level and made part of the CTile class instead. -TDP
    if (m_Pics[nPicNum].frames > 1 ) {
        m_Pics[nPicNum].currentframe = (m_Ticks / m_Pics[nPicNum].delay)
                                        % m_Pics[nPicNum].frames;
    }

	// Translate the coordinates into a format suitable to an SDL blit.
	// The above code could be rewritten so this is not necissary. -TDP
    srcrect.x = rect.left;
    srcrect.y = rect.top;
    srcrect.w = rect.right-rect.left;
    srcrect.h = rect.bottom-rect.top;
    dstrect.x = xpos;
    dstrect.y = ypos;
    dstrect.w = rect.right-rect.left;
    dstrect.h = rect.bottom-rect.top;
 
	// Check for any special effects
    if (nDrawType >= DP_HILIGHT) {
		// What is the bit depth of the display
        switch(m_screen->format->BytesPerPixel) {
			// Call TranPic8 for 256 color (8 bit color depth)
            case 1: TranPic8( &srcrect, &dstrect, nDrawType ); break;
			// Call TanPic16 for 65536 colors (16 bit color depth)
            case 2: TranPic16( &srcrect, &dstrect, nDrawType ); break;
        }
    }
	// Do a standard blit operation
    else if ( SDL_BlitSurface(m_image, &srcrect, m_screen, &dstrect) < 0 ) {
        SDL_FreeSurface(m_image);
        fprintf(stderr, "Problem: %s\n", SDL_GetError());
        exit(1);
    }

    return 0;
}




//-------------------------------------------------------------------------
// Method:    TranPic8
// Paramters: srcrect = the coordinates within the offscreen image to
//                      copy from.
//            dstrect = the coordinates within the screen surface to
//                      copy to.
//            nDrawType = type of draw opperation to do (straight blit,
//                        shadow effect, highlight, ...
// Returns:   0 on success, 1 on failure
//
// This function does a custom blit on 8 bit (256 color) screens.  It
// does several funky special effects, but uses direct buffer memory
// access to accomplish it, so it is slower than a standard hardware
// accellerated blit.  You should use these effects sparingly or risk
// slowing down your game.
//-------------------------------------------------------------------------
int CGridView::TranPic8(SDL_Rect *srcrect, SDL_Rect *dstrect, int nDrawType)
{
    int top, bottom, left, right;
    Uint8 *srcbuf, *dstbuf;
    Uint8 trancolor;
    Uint8 hicolor, pixel;
    int x, y;

    if ( SDL_MUSTLOCK(m_screen) ) {
        if ( SDL_LockSurface(m_screen) < 0 ) {
            fprintf(stderr, "Problem: %s\n", SDL_GetError());
            return 1;
        }
    }
    if ( SDL_MUSTLOCK(m_image) ) {
        if ( SDL_LockSurface(m_image) < 0 ) {
            fprintf(stderr, "Problem: %s\n", SDL_GetError());
            return 1;
        }
    }

    srcbuf = (Uint8 *) m_image->pixels;
    dstbuf = (Uint8 *) m_screen->pixels;

    trancolor = SDL_MapRGB(m_image->format, 255,255,255 );
    hicolor = SDL_MapRGB(m_image->format, 0,255,0 );
 
    // We should adjust for the edge of the screen! -TDP;
    left = srcrect->x;
    right = srcrect->x+srcrect->w-1;
    top = srcrect->y;
    bottom = srcrect->y+srcrect->h-1;

    for (y=top; y<bottom; y++) {
        for (x=left; x<right; x++) {
            pixel = srcbuf[y*m_image->pitch + x];
            if (pixel != trancolor) {
                if (nDrawType == DP_HILIGHT) {
                    if ((x+y)%2) {
                        dstbuf[(y-top+dstrect->y)*m_screen->pitch +
                            x-left+dstrect->x] = hicolor;
                    }
                    else {
                        dstbuf[(y-top+dstrect->y)*m_screen->pitch +
                            x-left+dstrect->x] = pixel;
                    }
                }
                else if (nDrawType == DP_SHADOW) {
                    pixel = dstbuf[(y-top+dstrect->y)*m_screen->pitch +
                        x-left+dstrect->x];
                    dstbuf[(y-top+dstrect->y)*m_screen->pitch +
                        x-left+dstrect->x] = m_ShadowColors[pixel];
                }
            }
            else if (nDrawType == DP_HILIGHT || nDrawType == DP_OUTLINE) {
                if ((x<right && srcbuf[y*m_image->pitch+x+1] != trancolor)
                  || (x>left && srcbuf[y*m_image->pitch+x-1] != trancolor)
                  || (y<bottom && srcbuf[(y+1)*m_image->pitch+x] != trancolor)
                  || (y>top && srcbuf[(y-1)*m_image->pitch+x] != trancolor)) {
                    dstbuf[(y-top+dstrect->y)*m_screen->pitch +
                        x-left+dstrect->x] = hicolor;
                }
            }
        }
    }

    if ( SDL_MUSTLOCK(m_screen) ) {
        SDL_UnlockSurface(m_screen);
    }
    if ( SDL_MUSTLOCK(m_image) ) {
        SDL_UnlockSurface(m_image);
    }

    return 0;
}



//-------------------------------------------------------------------------
// Method:    TranPic16
// Paramters: srcrect = the coordinates within the offscreen image to
//                      copy from.
//            dstrect = the coordinates within the screen surface to
//                      copy to.
//            nDrawType = type of draw opperation to do (straight blit,
//                        shadow effect, highlight, ...
// Returns:   0 on success, 1 on failure
//
// This function does a custom blit on 16 bit (65536 color) screens.  It
// does several funky special effects, but uses direct buffer memory
// access to accomplish it, so it is slower than a standard hardware
// accellerated blit.  You should use these effects sparingly or risk
// slowing down your game.
//-------------------------------------------------------------------------
int CGridView::TranPic16(SDL_Rect *srcrect, SDL_Rect *dstrect, int nDrawType)
{
    int top, bottom, left, right;
    Uint16 *srcbuf, *dstbuf;
    Uint16 trancolor;
    Uint16 hicolor, pixel;
    Uint32 newpixel;
    int x, y;
    int pitch;

    if ( SDL_MUSTLOCK(m_screen) ) {
        if ( SDL_LockSurface(m_screen) < 0 ) {
            fprintf(stderr, "Problem: %s\n", SDL_GetError());
            return 1;
        }
    }
    if ( SDL_MUSTLOCK(m_image) ) {
        if ( SDL_LockSurface(m_image) < 0 ) {
            fprintf(stderr, "Problem: %s\n", SDL_GetError());
            return 1;
        }
    }

    srcbuf = (Uint16 *) m_image->pixels;
    dstbuf = (Uint16 *) m_screen->pixels;

    trancolor = SDL_MapRGB(m_image->format, 255,255,255 );
    hicolor = SDL_MapRGB(m_image->format, 0,255,0 );
 
    // We should adjust for the edge of the screen! -TDP;
    left = srcrect->x;
    right = srcrect->x+srcrect->w-1;
    top = srcrect->y;
    bottom = srcrect->y+srcrect->h-1;
    pitch = m_screen->pitch / 2;

    for (y=top; y<bottom; y++) {
        for (x=left; x<right; x++) {
            pixel = srcbuf[y*pitch + x];
            if (pixel != trancolor) {
                if (nDrawType == DP_HILIGHT) {
                    if ((x+y)%2) {
                        dstbuf[(y-top+dstrect->y)*pitch +
                            x-left+dstrect->x] = hicolor;
                    }
                    else {
                        dstbuf[(y-top+dstrect->y)*pitch +
                            x-left+dstrect->x] = pixel;
                    }
                }

                else if (nDrawType == DP_SHADOW) {
                    pixel = dstbuf[(y-top+dstrect->y)*pitch +
                        x-left+dstrect->x];
                    newpixel = (((pixel & 0xf800)*3/4) & 0xf800) |
                               (((pixel & 0x07e0)*3/4) & 0x07e0) |
                               (((pixel & 0x001f)*3/4) & 0x001f);
                    dstbuf[(y-top+dstrect->y)*pitch +
                        x-left+dstrect->x] = (Uint16) newpixel;
                }
            }
            else if (nDrawType == DP_HILIGHT || nDrawType == DP_OUTLINE) {
                if ((x<right && srcbuf[y*pitch+x+1] != trancolor)
                  || (x>left && srcbuf[y*pitch+x-1] != trancolor)
                  || (y<bottom && srcbuf[(y+1)*pitch+x] != trancolor)
                  || (y>top && srcbuf[(y-1)*pitch+x] != trancolor)) {
                    dstbuf[(y-top+dstrect->y)*pitch +
                        x-left+dstrect->x] = hicolor;
                }
            }
        }
    }

    if ( SDL_MUSTLOCK(m_screen) ) {
        SDL_UnlockSurface(m_screen);
    }
    if ( SDL_MUSTLOCK(m_image) ) {
        SDL_UnlockSurface(m_image);
    }

    return 0;
}



//-------------------------------------------------------------------------
// Method:    Initialize
// Paramters: none
// Returns:   0 on success, 1 on failure
//
// This function initialize the video and audio hardware.
//-------------------------------------------------------------------------
int CGridView::Initialize( void )
{
    int i;
    int k=0;
    Uint8 red, green, blue;
    int usesound = 1;
 
    /* Initialize SDL */
    if (SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0 ) {
        if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
            fprintf(stderr, "Problem: %s\n", SDL_GetError());
            exit(1);
        }
        usesound = 0;
        printf("Sound support has been disabled.\n");
    }
    atexit(SDL_Quit);

    /* Load a BMP image into a surface */
    m_imagebmp = SDL_LoadBMP(m_strBitmap);
    if ( m_imagebmp == NULL ) {
        fprintf(stderr, "Problem: %s\n", SDL_GetError());
        exit(1);
    }

    /* Set the video mode (640x480 at native depth) */
    m_screen = SDL_SetVideoMode(640, 480, 0, SDL_HWSURFACE|SDL_HWPALETTE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
    if ( m_screen == NULL ) {
        fprintf(stderr, "Problem: %s\n", SDL_GetError());
        exit(1);
    }
    printf("debug 1\n");

    /* Set the video colormap */
    if ( m_imagebmp->format->palette != NULL ) {
        SDL_SetColors(m_screen,
        m_imagebmp->format->palette->colors, 0,
        m_imagebmp->format->palette->ncolors);
        printf("setting up shadow palette\n");
        for (i=0; i<256; i++) {
            red = m_imagebmp->format->palette->colors[i].r / 4 * 3;
            green = m_imagebmp->format->palette->colors[i].g / 4 * 3;
            blue = m_imagebmp->format->palette->colors[i].b / 4 * 3;
            m_ShadowColors[i]= (Uint8) SDL_MapRGB(
                m_imagebmp->format, red, green, blue );
        }
    }

    /* Convert the image to the video format (maps colors) */
    m_image = SDL_DisplayFormat(m_imagebmp);
    SDL_FreeSurface(m_imagebmp);
    if ( m_image == NULL ) {
        fprintf(stderr, "Problem: %s\n", SDL_GetError());
        exit(1);
    }

    // Set the transparent color
    SDL_SetColorKey(m_image, SDL_SRCCOLORKEY,
         SDL_MapRGB(m_image->format, 255,255,255));

    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
    SDL_ShowCursor(0);

    // Initialize the audiolist
    for (i=0; i<MAX_AUDIO_EVENTS; i++) {
        m_AudioQueue[i].len = 0;
    }

    if (usesound) {
        m_AudioSpec.freq = 22050;
        m_AudioSpec.format = AUDIO_S16;
        m_AudioSpec.channels = 2;
        m_AudioSpec.samples = 1024;
        m_AudioSpec.callback = fill_audio;
        m_AudioSpec.userdata = m_AudioQueue;
 
        if ( SDL_OpenAudio(&m_AudioSpec, NULL) < 0 ) {
            fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
        }
        else {
            // Load sounds
            LoadSounds("sounds.dat");  // this should not be hardcoded -TDP
            SDL_PauseAudio(0);
        }
    }

    printf("Bytes per pixel = %d\n", m_screen->format->BytesPerPixel);
    printf("Init done\n");

    return 0;
}






void CGridView::SetActive( int bParam )
{
    m_Active = bParam;
}




int CGridView::IsActive()
{
    return m_Active;
}




void CGridView::ClearBackground()
{
    ClearBox(m_ViewLeftBorder, m_ViewTopBorder,
        m_ViewRightBorder, m_ViewBottomBorder);
}




void CGridView::ClearBox(int left, int top, int right, int bottom)
{
    SDL_Rect rect;
    rect.x = left;
    rect.y = top;
    rect.w = right-left;
    rect.h = bottom-top;
    SDL_FillRect(m_screen, &rect, SDL_MapRGB(m_screen->format, 0, 0, 0));
}



void CGridView::DrawString( int xpos, int ypos, char *str )
{
    int k=0;
    int ch;
    int fontwidth=8;
    SDL_Rect srcrect;
    SDL_Rect dstrect;

    while (str[k]) {
        ch = str[k];
        if (ch > 31 && ch < 160) {
            srcrect.x = (ch%32)*8 + 3;
            srcrect.y = (ch/32)*15 + 146;
            srcrect.w = 8;
            srcrect.h = 15;
            dstrect.x = xpos + k*8;
            dstrect.y = ypos;
            dstrect.w = 8;
            dstrect.h = 15;
            if ( SDL_BlitSurface(m_image, &srcrect, m_screen, &dstrect) < 0 ) {
                SDL_FreeSurface(m_image);
                fprintf(stderr, "Problem: %s\n", SDL_GetError());
                exit(1);
            }
        }
        k++;
    }
}




void CGridView::DrawLine( int x1, int y1, int x2, int y2)
{
}





int CGridView::ShowMouse( int nXpos, int nYpos )
{
    return 0;
}







int CGridView::InputPending( void )
{
    return SDL_PollEvent(NULL);
}




int CGridView::GetInput( void )
{
    SDL_Event event;

    if (SDL_PollEvent(&event)) {
        if (event.type == SDL_KEYDOWN) {
            return event.key.keysym.sym;
        }
        else if (event.type ==  SDL_MOUSEMOTION ) { 
             m_MouseX = event.motion.x;
             m_MouseY = event.motion.y; 
        }
        else if (event.type == SDL_MOUSEBUTTONUP ) {
             return GS_CURSOR_UP;
        }
    }
    return 0;
}





void CGridView::GetMousePos( int *xpos, int *ypos )
{
   *xpos = m_MouseX;
   *ypos = m_MouseY;
}





//-------------------------------------------------------------------------
// Method:    LoadPics
// Paramters: filename
// Returns:   The number of images loaded into the image cache
//
// This function reads in a file containing a list of all the images to
// store in the offscreen image cache.  Currently it just loads the
// position info for all the sub-pictures in one big BMP file that is
// loaded in the Initialize method.
//
// In the future it will accept the names of individual image files.
// Perhaps the filename parameter could optionally be a URL, and
// indivual images defined in the file could likewise be loaded over
// the net.
//-------------------------------------------------------------------------
int CGridView::LoadPics(char *filename)
{
    FILE *fh;
    int k=0;
    char buf[1024];
    int num;

	// open the file for reading
    fh = fopen(filename, "r");
    if (!fh) {
        printf("Unable to open picture file\n");
        return 0;
    }
	// read the first line
    fgets(buf, 1024, fh);
	// loop until the end of the file
    while (!feof(fh)) {
		// read in the coordinates of the image within the image cache
        sscanf(buf, "%d %d %d %d %d %d %d %d %d", &num,
            &(m_Pics[k].left),
            &(m_Pics[k].top), 
            &(m_Pics[k].right), 
            &(m_Pics[k].bottom), 
            &(m_Pics[k].height), 
            &(m_Pics[k].frames), 
            &(m_Pics[k].currentframe), 
            &(m_Pics[k].delay) );
        k++;
        fgets(buf, 1024, fh);  // read the next line
    }
    m_TotalPics = k;

    return m_TotalPics; // return the total pictures loaded
}




//-------------------------------------------------------------------------
// Method:   LoadSounds
// Paramters: filename
// Returns:   The number of sound files loaded
//
// This function reads in a file containing a list of all the sound effect
// WAV files and loads each sound into RAM.
//
// Like the LoadPics call, this function could eventually be extended to
// accepted a URL instead of a file name, and the individual WAV files
// could likewise be expressed as URLs.  It could also be extended to
// load other sound formats (such as MP3 or Vorbis).
//-------------------------------------------------------------------------
int CGridView::LoadSounds(char *filename)
{
    SDL_AudioSpec spec;
    FILE *fh;
    int k=0;
    char buf[256];
    char name[256];
    char wavfile[256];
    int num;
 
    // Load the sound file list
    fh = fopen(filename, "r");
    if (!fh) {
        printf("Unable to open picture file\n");
        return 1;
    }
    
    // Get the first line of the file
    fgets(buf, sizeof(buf)-1, fh);
    // Loop until we reach the end of the file
    while (!feof(fh)) {
	// Parse the line
        sscanf(buf, "%d %s %s", &num, name, wavfile );
	// Load the WAV file
        SDL_LoadWAV(wavfile, &spec,
            &(m_Sounds[k].buf), &(m_Sounds[k].len) );
        k++;  // Increment the counter
        // Get the next line in the file
        fgets(buf, sizeof(buf)-1, fh);
    }
    m_TotalSounds = k;  // record the total number of loaded sounds

    return k; // return the number of loaded sounds
}




//-------------------------------------------------------------------------
// Method:  GetPic
// Paramters: PicNum = the integer sequence number of a loaded picture 
// Returns:   A pointer to the GPic structure of the specified picture
//-------------------------------------------------------------------------
GPic* CGridView::GetPic( int PicNum )
{
    return &m_Pics[PicNum];
}





//-------------------------------------------------------------------------
// Method:   PlaySounds
// Paramters: SoundNum = the integer sequence number of a loaded sound 
// Returns:   0 on success, 1 on failure
//
// This function queues up a sound for playing.
//-------------------------------------------------------------------------
int CGridView::PlaySound( int SoundNum )
{
    int i;

    // Loop through all possible audio slots
    for (i=0; i<MAX_AUDIO_EVENTS; i++) {
	// If a slot has a zero length, then it is empty and may be used
        if (m_AudioQueue[i].len == 0) {
            // Load the data into the audio slot	
            m_AudioQueue[i].len = m_Sounds[SoundNum].len;
            m_AudioQueue[i].pos = m_Sounds[SoundNum].buf;
	    // No need to keep looping; exit with non error status.
            return 0;
        }
    }

    // No empty slots were found; exit with error status
    return 1;
}




//-------------------------------------------------------------------------
// fill_audio is a function called periodically by the SDL framework.
// It spools playing sounds to the audio device, mixing multiple
// sounds as needed.
//-------------------------------------------------------------------------
void fill_audio(void *udata, Uint8 *stream, int len)
{
    int i;
    int mixlen;
    GAudioEvent *audiolist = (GAudioEvent*) udata;

    // Loop through all audio slots
    for (i=0; i<MAX_AUDIO_EVENTS; i++) {
	    // If the length is greater than zero, there is
	    // audio to be played in this slot
        if ( audiolist[i].len > 0 ) {
	        // Determine length of audio data to mix
            mixlen = ( len > (int) audiolist[i].len ? (int) audiolist[i].len : len );
	        // Mix the audio
            SDL_MixAudio(stream, audiolist[i].pos, mixlen, SDL_MIX_MAXVOLUME);
	        // Adjust our position in this audio slot
            audiolist[i].pos += mixlen;
	        // Adjust the length of remaining audio
            audiolist[i].len -= mixlen;
        }
    }
}

