/*------------------------------------------------------------------------
droidwar.cpp: Example video game for the GridSlammer toolkit
DroidWars Video Game - version 0.1

    Copyright (C) 2000 Thad D. Phetteplace

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Examine the license.txt file located in the main source code directory
    to see the full text of the GPL license document.

    Direct inquiries to:    The GridSlammer Project
                            P.O. Box 26354
                            Wauwatosa, WI 53208

    Or on the Internet at:  http://www.gridslammer.org
                            info@gridslammer.org
------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "GridSlammer/Grid.h"
#include "DroidStage.h"
#include "Droid.h"
#include "Door.h"
#include "GridMenu.h"

#ifdef TARGET_GGI

#include <ggi/ggi.h>
#include <ggi/keyboard.h>

ggi_color black  = { 0x0000, 0x0000, 0x0000 };
ggi_color white  = { 0xffff, 0xffff, 0xffff };
ggi_color blue   = { 0x0000, 0x0000, 0xffff };
ggi_color yellow = { 0xffff, 0xffff, 0x0000 };

#define GS_KEY_ESC          GIIUC_Escape
#define GS_KEY_LEFT_ARROW   GIIK_Left
#define GS_KEY_RIGHT_ARROW  GIIK_Right
#define GS_KEY_UP_ARROW     GIIK_Up
#define GS_KEY_DOWN_ARROW   GIIK_Down
#define GS_KEY_SPACEBAR     GIIUC_Space
#define GS_KEY_ENTER        GIIUC_Return

#else

#define GS_KEY_ESC          SDLK_ESCAPE
#define GS_KEY_LEFT_ARROW   SDLK_LEFT
#define GS_KEY_RIGHT_ARROW  SDLK_RIGHT
#define GS_KEY_UP_ARROW     SDLK_UP
#define GS_KEY_DOWN_ARROW   SDLK_DOWN
#define GS_KEY_SPACEBAR     SDLK_SPACE
#define GS_KEY_ENTER        SDLK_RETURN

#endif

CDroidStage GridStage;
CGridView GridView;
CGridMenu Menu;
CDroid* MyDroid;
CDroid* TempDroid;
GGridInfo gridinfo;


int main(int argc, char *argv[])
{
    time_t starttime, stoptime;
    int seconds;
    int frames = 0;
    int action;
    int quitgame = 0;

    srand(1);
    GridView.Initialize();
    GridStage.LoadTiles( &GridView, "nofile");
    
    printf("Loading Grid\n"); fflush(stdout);

    // Greate a method for filling all this in!  -TDP
    gridinfo.Grid = GridStage.m_Grid;
    gridinfo.GridObjs = GridStage.m_GridObjs;
    gridinfo.Tiles = GridStage.m_Tiles;
    gridinfo.Objects = GridStage.m_Objects;
    gridinfo.TotalGridObjs = &(GridStage.m_TotalGridObjs);
    gridinfo.Sounds = GridView.m_Sounds;
    //gridinfo.TotalSounds = 0;
    gridinfo.AudioQueue = GridView.m_AudioQueue;

    GridStage.LoadGrid( "grid1.grd", &gridinfo );
    MyDroid = (CDroid *) GridStage.m_Objects[0];  // fix this! -TDP

    starttime = time(0);
    printf("About to enter the rendering loop\n"); fflush(stdout);
    while( !quitgame )
    {
        frames++;

        //if (frames > 10000) quitgame = 1; // Delete this! -TDP

        if (Menu.IsActive()) {
            Menu.Render();
        }
        else {
            GridStage.Process();
            GridStage.Render(&GridView);
        }
        if (GridView.InputPending()) {
            while (GridView.InputPending())
                action = GridView.GetInput();
            if (Menu.IsActive()) {
                switch(action)
                {
                case GS_KEY_ESC:
                    Menu.Close();
                    break;
                case GS_KEY_UP_ARROW:
                    Menu.ArrowUp();
                    break;
                case GS_KEY_DOWN_ARROW:
                    Menu.ArrowDown();
                    break;
                case GS_KEY_SPACEBAR:
                case GS_KEY_ENTER:
                    switch(Menu.Select()) {
                        case 0: break;
                        case 1: break;
                        case 2: break;
                        case 3: break;
                        case 4: break;
                        case 5: quitgame = 1; break;
                    }
                    break;
                }
            }
            else {
                switch(action) {
                case GS_CURSOR_UP:
                    if (GridStage.GetGridTile((int) GridStage.m_CursorX,
                        (int) GridStage.m_CursorY) == 0 ) {
                        GridStage.SetGridTile((int) GridStage.m_CursorX,
                            (int) GridStage.m_CursorY, 25);
                    } 
                    else {
                        GridStage.SetGridTile((int) GridStage.m_CursorX,
                            (int) GridStage.m_CursorY, 0 );
                    }
                    break;
                case GS_KEY_UP_ARROW:
                    MyDroid->ChangeSpeed( 0.05 );
                    break;
                case GS_KEY_DOWN_ARROW:
                    MyDroid->ChangeSpeed( -0.05 );
                    break;
                case GS_KEY_LEFT_ARROW:
                    MyDroid->ChangeDirection( 0.1 );
                    break;
                case GS_KEY_RIGHT_ARROW:
                    MyDroid->ChangeDirection( -0.1 );
                    break;
                case GS_KEY_SPACEBAR:
                    MyDroid->FireWeapon( 1 );
                    break;
                case GS_KEY_ESC:
                    GridStage.Render(&GridView);
                    GridStage.Render(&GridView);
                    Menu.Open(&GridView);
                    break;
                }
            }
        }
    }
    stoptime = time(0);
    seconds = (int) (stoptime - starttime);
    printf("Frames per second = %d\n", frames / seconds );

    return 0;
}

